@@ -1,9 +1,23 @@
 from pyboof.geo import *
 from pyboof.calib import *
 from pyboof import gateway
+from pyboof import MmapType
 from py4j.java_collections import ListConverter
 import tempfile
 
+
+def string_to_bytearray(message: str):
+    """
+    Converts a string into a bytearray. This effectively does a 'static_cast' of the String into byte[]. Useful when
+    a marker encodes a binary format such as a zip file
+    """
+    raw_data = bytearray(len(message))
+    for i in range(len(raw_data)):
+        raw_data[i] = ord(message[i]) % 256
+        # If the unicode value is outside the 8-bit range the data is likely to be corrupted... but wont crash@
+    return raw_data
+
+
 class ConfigPolygonDetector(JavaConfig):
     def __init__(self):
         JavaConfig.__init__(self, "boofcv.struct.Configuration.ConfigPolygonDetector")
@@ -103,6 +117,11 @@ class ConfigMicroQrCode(JavaConfig):
         JavaConfig.__init__(self, "boofcv.factory.fiducial.ConfigMicroQrCode")
 
 
+class ConfigAztecCode(JavaConfig):
+    def __init__(self):
+        JavaConfig.__init__(self, "boofcv.factory.fiducial.ConfigAztecCode")
+
+
 class ConfigUchiyaMarker(JavaConfig):
     def __init__(self):
         JavaConfig.__init__(self, "boofcv.factory.fiducial.ConfigUchiyaMarker")
@@ -270,8 +289,12 @@ class QrCode:
 
     def __init__(self, java_object=None):
         if java_object is None:
-            self.verson = -1
+            self.version = -1
             self.message = ""
+            self.corrected = None  # raw byte data after error correction
+            self.byteEncoding = ""  # Encoding used when in BYTE mode
+            self.totalBitErrors = -1  # Number of errors detected in error correction
+            self.bitsTransposed = False  # true if the QR code is transposed
             self.error_level = ""
             self.mask_pattern = ""
             self.mode = ""
@@ -282,8 +305,12 @@ class QrCode:
             self.pp_down = Polygon2D(4)
         else:
             jobj = JavaWrapper(java_object)
-            self.verson = jobj.version
+            self.version = jobj.version
             self.message = jobj.message
+            self.corrected = mmap_array_java_to_python(jobj.corrected, MmapType.ARRAY_U8)
+            self.byteEncoding = jobj.byteEncoding
+            self.totalBitErrors = jobj.totalBitErrors
+            self.bitsTransposed = jobj.bitsTransposed
             self.error_level = jobj.error.toString()
             self.mask_pattern = jobj.mask.toString()
             self.mode = jobj.mode.toString()
@@ -326,8 +353,12 @@ class MicroQrCode:
 
     def __init__(self, java_object=None):
         if java_object is None:
-            self.verson = -1
+            self.version = -1
             self.message = ""
+            self.corrected = None  # raw byte data after error correction
+            self.byteEncoding = ""  # Encoding used when in BYTE mode
+            self.totalBitErrors = -1  # Number of errors detected in error correction
+            self.bitsTransposed = False  # true if the QR code is transposed
             self.error_level = ""
             self.mask_pattern = ""
             self.mode = ""
@@ -336,8 +367,12 @@ class MicroQrCode:
             self.pp = Polygon2D(4)
         else:
             jobj = JavaWrapper(java_object)
-            self.verson = jobj.version
+            self.version = jobj.version
             self.message = jobj.message
+            self.corrected = mmap_array_java_to_python(jobj.corrected, MmapType.ARRAY_U8)
+            self.byteEncoding = jobj.byteEncoding
+            self.totalBitErrors = jobj.totalBitErrors
+            self.bitsTransposed = jobj.bitsTransposed
             self.error_level = jobj.error.toString()
             self.mask_pattern = jobj.mask.toString()
             self.mode = jobj.mode.toString()
@@ -354,7 +389,7 @@ class MicroQrDetector(JavaWrapper):
 
     Attributes:
         detections: List of detected MicroQrCode
-        failures: List of objects that are highly likely to be a QR Code but were rejected.
+        failures: List of objects that are highly likely to be a Micro QR Codes but were rejected.
     """
 
     def __init__(self, java_detector):
@@ -371,6 +406,61 @@ class MicroQrDetector(JavaWrapper):
         return ImageType(self.java_obj.getImageType())
 
 
+class AztecCode:
+    """Description of a detected Aztec Code inside an image.
+
+    """
+
+    def __init__(self, java_object=None):
+        if java_object is None:
+            self.dataLayers = 0
+            self.messageWordCount = 0
+            self.rawbits = None  # raw byte data
+            self.corrected = None  # raw byte data after error correction
+            self.message = ""
+            self.structure = ""
+            self.failure = ""
+            self.transposed = False   # true if the marker is transposed
+            self.totalBitErrors = -1  # Number of errors detected in error correction
+            self.bounds = Polygon2D(4)
+        else:
+            jobj = JavaWrapper(java_object)
+            self.dataLayers = jobj.dataLayers
+            self.messageWordCount = jobj.messageWordCount
+            self.rawbits = mmap_array_java_to_python(jobj.rawbits, MmapType.ARRAY_U8)
+            self.corrected = mmap_array_java_to_python(jobj.corrected, MmapType.ARRAY_U8)
+            self.message = jobj.message
+            self.structure = jobj.structure.toString()
+            self.transposed = jobj.transposed
+            self.totalBitErrors = jobj.totalBitErrors
+            self.bounds = Polygon2D(jobj.bounds)
+
+            if jobj.failure is not None:
+                self.failure_cause = jobj.failure.toString()
+
+
+class AztecCodeDetector(JavaWrapper):
+    """Detects Aztec Codes inside of images
+
+    Attributes:
+        detections: List of detected AztecCode
+        failures: List of objects that are highly likely to be Aztec Codes but were rejected.
+    """
+
+    def __init__(self, java_detector):
+        JavaWrapper.__init__(self, java_detector)
+        self.detections = []
+        self.failures = []
+
+    def detect(self, image):
+        self.java_obj.process(image)
+        self.detections = [AztecCode(x) for x in self.java_obj.getDetections()]
+        self.failures = [AztecCode(x) for x in self.java_obj.getFailures()]
+
+    def get_image_type(self):
+        return ImageType(self.java_obj.getImageType())
+
+
 class UchiyaRandomDotDetector(FiducialDetector):
     """Detector for random dot markers
     """
@@ -621,6 +711,20 @@ class FactoryFiducial:
             microqr(jconf, self.boof_image_type)
         return MicroQrDetector(java_detector)
 
+    def aztec(self, config: ConfigAztecCode = None) -> AztecCodeDetector:
+        """ Creates a detector for QR Codes
+
+        :param config: ConfigQrCode or None
+        :return: QrCodeDetector
+        """
+        if config is None:
+            jconf = None
+        else:
+            jconf = config.java_obj
+
+        java_detector = gateway.jvm.boofcv.factory.fiducial.FactoryFiducial. \
+            aztec(jconf, self.boof_image_type)
+        return AztecCodeDetector(java_detector)
 
     def random_dots(self, config: ConfigUchiyaMarker) -> UchiyaRandomDotDetector:
         """ Creates a detector random dot / Uchiya markers
